/**************************************
 * $Author: kaneda $
 * $Date: 2003/02/12 12:45:17 $
 **************************************/

/* ph_msg.h */

#ifndef __PH_MSG_H__
#define __PH_MSG_H__

#include "ph_util.h"
#include "ph_vp.h"
#include "ph_conn.h"



/* Macro Values */
enum {
  PH_MSG_HEADER_SIZE 		= 256
};

#define 			PH_SLIDING_WINDOW_ON

/* Structures */

enum ph_msg_meta_type {
  invalid, routing, view
#ifdef PH_ROUTING_ENABLE_DOMINATING
  ,conn_type, conn_type_reply
#endif
};

typedef enum ph_msg_meta_type	ph_msg_meta_type_t;

enum ph_msg_type {
  meta, user
};

typedef enum ph_msg_type	ph_msg_type_t;

struct ph_msg {
  ph_msg_type_t 	type;
  ph_vp_t		dest;

  ph_conn_t		to_conn, from_conn;

  ph_msg_meta_type_t	meta_type;
  void * 		body;
  size_t		len;
  int			tag;

  pthread_mutex_t	mp;		/* to synchronize access to variables */
};

typedef struct ph_msg	* ph_msg_t;


PH_DEFINE_LIST_STRUCTURE(ph_msg) 


/* Public Function Prototypes */
ph_msg_t ph_msg_new(ph_msg_type_t type, ph_vp_t dest, void * body, size_t len, int tag);
void ph_msg_dispose(ph_msg_t msg);
void ph_msg_print(ph_msg_t msg);

ph_msg_t ph_msg_new_user(ph_vp_t dest, void * body, size_t len, int tag);
ph_msg_t ph_msg_new_meta(ph_conn_t conn, ph_msg_meta_type_t meta_type, void * body, size_t len);

int ph_msg_send(ph_msg_t msg);
ph_msg_t ph_msg_recv(ph_conn_t conn);

int ph_msg_is_remote_msg(ph_msg_t msg, fd_set * w_fdset_p);
int ph_msg_is_local_user_msg(ph_msg_t msg, ph_vp_set_sync_t vp_set, int tag);
int ph_msg_is_local_meta_msg(ph_msg_t msg, ph_vp_set_sync_t vp_set);


PH_DEFINE_LIST_FUNC_PROTOTYPE(ph_msg) 

void ph_msg_list_print(ph_msg_list_t msg_list);
ph_msg_t ph_msg_list_dequeue_remote_msg(ph_msg_list_t msg_list, fd_set * w_fdset_p);
ph_msg_t ph_msg_list_dequeue_local_user_msg(ph_msg_list_t msg_list, ph_vp_set_sync_t vp_set, int tag, int is_block);
ph_msg_t ph_msg_list_dequeue_local_meta_msg(ph_msg_list_t msg_list, ph_vp_set_sync_t vp_set);
void ph_msg_list_store(ph_msg_list_t msg_list, char * filename);
void ph_msg_list_load(ph_msg_list_t msg_list, char * filename);
int ph_msg_list_is_empty_user_msg(ph_msg_list_t msg_list);
void ph_msg_list_clear_conn(ph_msg_list_t msg_list, ph_conn_t conn);

#endif



