/**************************************
 * $Author: kaneda $
 * $Date: 2003/02/12 12:45:17 $
 **************************************/

/* ph_pt2pt.h -- reliable point-to-point communication on top of TCP/IP */


#ifndef __PH_PT2PT_H__
#define __PH_PT2PT_H__

#include "ph_util.h"
#include "ph_vp.h"
#include "ph_addr.h"
#include "ph_conn.h"
#include "ph_msg.h"
#include "ph_routing.h"

/* Macro Values */


#define PH_VIEWER

enum {
  PH_MAX_CONNECTION		= 256,
  PH_SELECT_TIMEOUT		= 500,
  PH_CONNECTOR_INTERVAL		= 1000,
  PH_FINALIZE_WAIT		= 1000,
  PH_ROUTER_FULL_UPDATE_INTERVAL= 20
};


/* ph_pt2pt_t is ...*/
struct ph_pt2pt {
  ph_vp_set_sync_t	vp_set;		/* current vp set */
  ph_vp_t		resource_vp;

  ph_msg_list_t		msg_list;	/* message is queued in this list. */

  ph_addr_list_t	addr_list;	/* list of hosts where the runtime may be running */
  ph_conn_list_t	conn_list;	/* list of connections that the runtime has established */
  ph_routing_t		routing; 	/* routing table */
  
  int			available_ports[2];
  int			listen_port;
  
  int 			is_finalizing;
  int 			is_disposed;
  pthread_mutex_t	mp;

  pthread_t 		sender_tid, receiver_tid, connector_tid, router_tid;

  int checker;

#ifdef PH_VIEWER
  int			is_viewing;
  int			viewer_update_interval;
  pthread_t		viewer_tid;
  ph_addr_t		viewer_addr;
  ph_conn_t		viewer_conn;
  char *		viewer_comment[100];
#endif
};

typedef struct ph_pt2pt	* ph_pt2pt_t;



/* Public Function Prototypes */
/* Followings are defined in pt2pt.c */
ph_pt2pt_t ph_pt2pt_new(int listen_port_lower, int listen_port_upper);
void ph_pt2pt_dispose(ph_pt2pt_t pt2pt);
void ph_pt2pt_print(ph_pt2pt_t pt2pt);
int ph_pt2pt_get_is_disposed(ph_pt2pt_t pt2pt);
void ph_pt2pt_set_is_disposed(ph_pt2pt_t pt2pt, int is_disposed);
int ph_pt2pt_get_is_finalizing(ph_pt2pt_t pt2pt);
void ph_pt2pt_set_is_finalizing(ph_pt2pt_t pt2pt, int is_finalizing);
#ifdef PH_VIEWER
int ph_pt2pt_get_is_viewing(ph_pt2pt_t pt2pt);
void ph_pt2pt_set_is_viewing(ph_pt2pt_t pt2pt, int is_viewing);
void ph_pt2pt_viewer_run(ph_pt2pt_t pt2pt, int viewer_update_interval);
#endif
void ph_pt2pt_run(ph_pt2pt_t pt2pt);
void ph_pt2pt_stop(ph_pt2pt_t pt2pt);


/* Followings are defined in user.c */
void ph_pt2pt_initialize(ph_pt2pt_t pt2pt);
void ph_pt2pt_assume_vp_set(ph_pt2pt_t pt2pt, ph_vp_set_t vp_set);
void ph_pt2pt_release_vp_set(ph_pt2pt_t pt2pt, ph_vp_set_t vp_set);
ph_vp_set_t ph_pt2pt_get_assumed_vps(ph_pt2pt_t pt2pt);
void ph_pt2pt_add_contact_point(ph_pt2pt_t pt2pt, char * addr, char * service, ph_addr_type_t addr_type, char * user);
void ph_pt2pt_send_user_msg(ph_pt2pt_t pt2pt, ph_vp_t dest, void * buf, size_t len, int tag);
ph_msg_t ph_pt2pt_recv_user_msg(ph_pt2pt_t pt2pt, int tag, int is_block);
void ph_pt2pt_invalidate_routing(ph_pt2pt_t pt2pt);
void ph_pt2pt_finalize(ph_pt2pt_t pt2pt);
int ph_pt2pt_messages_is_empty(ph_pt2pt_t pt2pt);
void ph_pt2pt_load_messages(ph_pt2pt_t pt2pt, char * filename);
void ph_pt2pt_store_messages(ph_pt2pt_t pt2pt, char * filename);
ph_vp_t ph_pt2pt_get_temp_vp();

#ifdef PH_VIEWER
void ph_pt2pt_viewer_initialize(ph_pt2pt_t pt2pt, int viewer_update_interval, char * addr, char * service, ph_addr_type_t addr_type, char * user);
void ph_pt2pt_viewer_put_comment(ph_pt2pt_t pt2pt, void * body, size_t len);
#endif

/* Followings are defined in meta.c */
ph_msg_t ph_pt2pt_create_meta_msg(ph_pt2pt_t pt2pt, ph_conn_t conn, ph_msg_meta_type_t meta_type, ...);
void ph_pt2pt_send_meta_msg(ph_pt2pt_t pt2pt, ph_msg_t msg);
void ph_pt2pt_recv_meta_msg(ph_pt2pt_t pt2pt, ph_msg_t msg);


/* Followings are defined in handler.c */
void ph_pt2pt_handle_connect(ph_pt2pt_t pt2pt, ph_addr_t addr);
void ph_pt2pt_handle_connecting(ph_pt2pt_t pt2pt, ph_conn_t conn, fd_set * r_fdset_p, fd_set * w_fdset_p);
void ph_pt2pt_handle_accepting(ph_pt2pt_t pt2pt, ph_conn_t conn, fd_set * r_fdset_p);
void ph_pt2pt_handle_opened(ph_pt2pt_t pt2pt, ph_conn_t conn, fd_set * r_fdset_p);
void ph_pt2pt_handle_error(ph_pt2pt_t pt2pt, ph_conn_t conn, fd_set * e_fdset_p);

void ph_pt2pt_handle_msg(ph_pt2pt_t pt2pt, ph_msg_t msg);




#endif
