/**************************************
 * $Author: kaneda $
 * $Date: 2003/02/12 12:45:17 $
 **************************************/

/* routing.h */

#ifndef __PH_ROUTING_H__
#define __PH_ROUTING_H__

#include "ph_util.h"
#include "ph_vp.h"
#include "ph_conn.h"
#include "ph_msg.h"


/* Macro Values */

enum {
  PH_MAX_METRIC			= 1000,
  PH_SETTLING_TIME_HISTORY_N	= 5
};


/* Structures */

struct ph_routing_entry {
  ph_vp_range_t	dest;		/* destination which is reachable via conn */
  ph_conn_t	conn;		/* prefered neighbor */
  ph_addr_type_t addr_type;

  int		metric;		/* distance between localhost and destination host */
  long		sequence;
  int		is_modified;

  struct timeval first;
  double 	st_time[PH_SETTLING_TIME_HISTORY_N];
  //int 		install_time;	/* to destroy stale messages */

};

typedef struct ph_routing_entry	* ph_routing_entry_t;


PH_DEFINE_LIST_STRUCTURE(ph_routing_entry) 


enum ph_routing_event_type {
  assume_vps, release_vps, repair, fail, update, finalize
};

typedef enum ph_routing_event_type	ph_routing_event_type_t;

struct ph_routing_event {
  ph_routing_event_type_t 	type;
  ph_conn_t 			from_conn;
  ph_routing_entry_list_t	entry_list;

#ifdef PH_ROUTING_ENABLE_DOMINATING
  int		is_gateway;
  
#endif
};

typedef struct ph_routing_event	* ph_routing_event_t;


PH_DEFINE_LIST_STRUCTURE(ph_routing_event) 


#ifdef PH_ROUTING_ENABLE_DOMINATING

struct ph_routing_neigh {
  ph_addr_t	addr;		/* for possible connection (addr) */
  ph_conn_t	conn;		/* for established connection */
  ph_vp_set_t	local;
  ph_vp_set_t 	neigh[2];	/* tcp, ssh ssh */
  ph_vp_set_t 	p_neigh[2];	/* possible neighbor (tcp, ssh) */
  struct timeval time;		/* time when it was updates */
};

typedef struct ph_routing_neigh	* ph_routing_neigh_t;


PH_DEFINE_LIST_STRUCTURE(ph_routing_neigh)
#endif


struct ph_routing {
  ph_routing_entry_list_t	entry_list;
  ph_routing_event_list_t 	event_list;

#ifdef PH_ROUTING_ENABLE_DOMINATING
  int				is_gateway;
  ph_routing_neigh_list_t	neigh_list;
#endif

  pthread_mutex_t		mp;
};
/* NOTE: to acquire pt2pt->routing->mp and pt2pt->conn_list->mp,
   you have to first acquire pt2pt->routing->mp and then pt2pt->conn_list->mp */

typedef struct ph_routing	* ph_routing_t;


/* Public Function Prototypes */
ph_routing_entry_t ph_routing_entry_new(ph_vp_t lower, ph_vp_t upper, ph_conn_t conn, int metric, int sequence, int is_modified, double st_time[], ph_addr_type_t addr_type);
void ph_routing_entry_dispose(ph_routing_entry_t p);


PH_DEFINE_LIST_FUNC_PROTOTYPE(ph_routing_entry)


ph_routing_entry_list_t ph_routing_entry_list_new2(ph_vp_set_t ph_vp_set);


ph_routing_event_t ph_routing_event_new(ph_routing_event_type_t type, ph_conn_t from_conn, ph_routing_entry_list_t entry_list);
void ph_routing_event_dispose(ph_routing_event_t p);


PH_DEFINE_LIST_FUNC_PROTOTYPE(ph_routing_event)

#ifdef PH_ROUTING_ENABLE_DOMINATING

ph_routing_neigh_t ph_routing_neigh_new(ph_conn_t conn);
void ph_routing_neigh_dispose(ph_routing_neigh_t p);
int ph_routing_neigh_is_same_conn(ph_routing_neigh_t p, void * arg1);

PH_DEFINE_LIST_FUNC_PROTOTYPE(ph_routing_neigh)

ph_routing_neigh_t ph_routing_neigh_list_find_same_addr(ph_routing_neigh_list_t neigh_list, ph_addr_t addr);

#endif

ph_routing_t ph_routing_new();
void ph_routing_dispose(ph_routing_t routing);
void ph_routing_print(ph_routing_t routing);

void ph_routing_handle_event(ph_routing_t routing, ph_routing_event_t event, int * is_inc_update_p, double * inc_update_time_p);
void ph_routing_route_enqueue_msg(ph_routing_t routing, ph_msg_list_t msg_list, ph_msg_t msg);
void ph_routing_route_msg_list(ph_routing_t routing, ph_msg_list_t msg_list);
double ph_routing_wait_event_list(ph_routing_t routing, double wait_time);
void ph_routing_increment_sequences(ph_routing_t routing);

char * ph_routing_create_msg(ph_routing_t routing, int * len_p, int is_full, int is_finalizing);
ph_routing_entry_list_t ph_routing_analyze_msg(char * entries);

#endif



 
