/**************************************
 * $Author: kaneda $
 * $Date: 2003/02/12 12:45:17 $
 **************************************/

/* ph_util.h */

#ifndef __PH_UTIL_H__
#define __PH_UTIL_H__


#ifdef __PH_WIN32__
  #define __MT__
#endif

#ifndef _REENTRANT
#define _REENTRANT
#endif


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h> /* ANSI C header file */
#include <assert.h>
#include <errno.h>
#include <signal.h>
#include <fcntl.h>
#include <ctype.h>
#include <limits.h>
#include <sys/types.h>	/* basic system data types */
#include <sys/stat.h>


#ifndef __PH_WIN32__
  #include <unistd.h>
  #include <strings.h>
  #include <netdb.h>
  #include <pthread.h>

  #include <sys/time.h>
  #include <sys/file.h>
  #include <sys/socket.h>	/* basic socket definitions */
  #include <sys/wait.h>
  #include <sys/un.h>		/* for Unix domain socket */
  #include <netinet/in.h>	/* sockaddr_in{} and other Internet defns */
  #include <netinet/in_systm.h>
  #include <netinet/ip.h>
  #include <netinet/tcp.h>
  #include <arpa/inet.h>	/* inet(3) functions */
#else
  #include <windows.h>
  #include <process.h>
  #include <winsock2.h>
#endif

#ifdef __PH_WIN32__

enum {
  PH_MAX_THREAD	= 1024
};

struct ph_cond {
  CRITICAL_SECTION	mp;
  int			front, rear;
  HANDLE		waiters[PH_MAX_THREAD];
};

typedef struct ph_cond	  ph_cond_t;


#define ssize_t		int
#define pid_t		int
#define pthread_t	HANDLE
#define pthread_mutex_t CRITICAL_SECTION
#define pthread_cond_t	ph_cond_t


#define ph_pthread_create(tid, attr, func, arg)	ph_begin_thread((tid), (func), (arg))
#define ph_pthread_join(tid, status)		ph_pthread_wait((tid))

#define ph_pthread_mutex_init(mptr, attr)	ph_initialize_critical_section((mptr))
#define ph_pthread_mutex_lock(mp)		ph_enter_critical_section((mp))
#define ph_pthread_mutex_unlock(mp)		ph_leave_critical_section((mp))
#define ph_pthread_mutex_destroy(mp)		ph_delete_critical_section((mp))

#define ph_pthread_cond_init(cptr, attr)	ph_cond_init((cptr))
#define ph_pthread_cond_destroy(cptr)		ph_cond_destroy((cptr))	
#define ph_pthread_cond_broadcast(cptr)		ph_cond_broadcast((cptr))
//#define ph_pthread_cond_signal(cptr)		ph_pthread_cond_signal(cptr)
#define ph_pthread_cond_wait(cptr, mptr)	ph_cond_wait((cptr), (mptr))
#define ph_pthread_cond_timedwait(cptr, mptr, tsptr)	ph_cond_wait((cptr), (mptr), (tsptr))


#endif





#ifndef INADDR_NONE
#define INADDR_NONE	0xffffffff	/* should have beeein in <netinet/in.h> */
#endif


/* Macro Values */
enum {
  PH_MAX_FD 	= 1024,
  PH_LISTENQ	= 1024,	/* nd argument to listen() */

  PH_MAX_LINE	= 4096,	/* max text line length */
  PH_BUFSIZE	= 8192
};


/* Basic Types */
typedef void ph_sigfunc(int);

/* Basic Types */
#ifndef HAVE_GETADDRINFO
#define socklen_t int
#endif



/* Structures */
/* ph_cell_t */
struct ph_cell {
  void 			* self;
  struct ph_cell 	* next;
};

typedef struct ph_cell	* ph_cell_t;

/* ph_list_t */
struct ph_list {
  ph_cell_t		head;
  ph_cell_t 		tail;
  int			nelm;

  int			is_sync;		/* if 1 then insert/delete of item is synchronized */

  pthread_mutex_t	mp;			/* synchronize insertion/detetion of a cell */
  pthread_cond_t	i_cond;			/* It is signaled when the new item is inserted to the list. */
  pthread_cond_t	d_cond;			/* It is signaled when the item is deleted. */
};

typedef struct ph_list	* ph_list_t;

typedef void (*ph_func1)(void *);
typedef int (*ph_func2)(void *, void *);
typedef void (*ph_func3)(void *, void *);
typedef int (*ph_func4)(void *, void *, void *);


/* Public Global Variables */


/* this is defined in error.c */
extern int ph_daemon_proc;



/* Public Function Prototypes */

/* Following functions are defined in debug.c */
void ph_debug_set_mode(int mode);
int ph_debug_get_mode();
void ph_debug_print(char * fmt, ...);
void ph_debug_print2(char * fmt, ...);
void ph_debug_print3(char * fmt, ...);
void ph_debug_print4(char * fmt, ...);
void ph_debug_print_message(pthread_mutex_t mp, char * buf, int len);

/* Following functions are defined in error.c */
void ph_err_ret(const char * fmt, ...);
void ph_err_sys(const char * fmt, ...);
void ph_err_dump(const char * fmt, ...);
void ph_err_msg(const char * fmt, ...);
void ph_err_quit(const char * fmt, ...);

/* Following functions are defined in wrap_unix.c */
void ph_free(void *ptr);
void * ph_malloc(size_t size);
void * ph_calloc(size_t n, size_t size);

#ifndef __PH_WIN32__
void ph_close(int fd);
void ph_dup(int fd);
void ph_dup2(int fd1, int fd2);
int ph_fcntl(int fd, int cmd, int arg);
void ph_gettimeofday(struct timeval *tv, void *foo);
int ph_ioctl(int fd, int request, void *arg);
pid_t ph_fork(void);
char * ph_getenv(const char *name);
void ph_mkstemp(char *temp);
int ph_open(const char *pathname, int oflag, mode_t mode);
void ph_pipe(int *fds);
ssize_t ph_read(int fd, void *ptr, size_t nbytes);
void ph_sigaddset(sigset_t *set, int signo);
void ph_sigdelset(sigset_t *set, int signo);
void ph_sigemptyset(sigset_t *set);
void ph_sigfillset(sigset_t *set);
int ph_sigismember(const sigset_t *set, int signo);
void ph_sigpending(sigset_t *set); 
void ph_sigprocmask(int how, const sigset_t *set, sigset_t *oset);
long ph_sysconf(int name);
void ph_unlink(const char *pathname);
pid_t ph_wait(int *iptr);
pid_t ph_waitpid(pid_t pid, int *iptr, int options);
void ph_write(int fd, void *ptr, size_t nbytes);
FILE * ph_fopen(const char * filename, const char * mode);
void ph_fclose(FILE * fp);
ph_sigfunc * ph_signal(int signo, ph_sigfunc * func);
#else
#endif


/* Following functions are defined in wrap_sock.c */
int ph_accept(int fd, struct sockaddr *sa, socklen_t *salenptr);
void ph_bind(int fd, const struct sockaddr *sa, socklen_t salen);
void ph_connect(int fd, const struct sockaddr *sa, socklen_t salen);
void ph_getpeername(int fd, struct sockaddr *sa, socklen_t *salenptr);
void ph_getsockname(int fd, struct sockaddr *sa, socklen_t *salenptr) ;
void ph_getsockopt(int fd, int level, int optname, void *optval, socklen_t *optlenptr);
void ph_listen(int fd, int backlog);
//ssize_t ph_recv(int fd, void *ptr, size_t nbytes, int flags);
//ssize_t ph_recvfrom(int fd, void *ptr, size_t nbytes, int flags, struct sockaddr *sa, socklen_t *salenptr);
//ssize_t ph_recvmsg(int fd, struct msghdr *msg, int flags);
int ph_select(int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout);
//void ph_send(int fd, const void *ptr, size_t nbytes, int flags);
//void ph_sendto(int fd, const void *ptr, size_t nbytes, int flags, const struct sockaddr *sa, socklen_t salen);
//void ph_sendmsg(int fd, const struct msghdr *msg, int flags);
void ph_setsockopt(int fd, int level, int optname, const void *optval, socklen_t optlen);
void ph_shutdown(int fd, int how);
int ph_socket(int family, int type, int protocol);
void ph_socketpair(int family, int type, int protocol, int *fd);
char * ph_sock_ntop(const struct sockaddr *sa, socklen_t salen);


/* Following functions are defined in wrap_pthread.c */
#ifndef __PH_WIN32__
void ph_pthread_create(pthread_t *tid, const pthread_attr_t *attr, void * (*func)(void *), void *arg);
void ph_pthread_join(pthread_t tid, void **status);
void ph_pthread_detach(pthread_t tid);
void ph_pthread_mutexattr_init(pthread_mutexattr_t *attr);
void ph_pthread_mutex_init(pthread_mutex_t *mptr, pthread_mutexattr_t *attr);
void ph_pthread_mutex_destroy(pthread_mutex_t *mptr);
void ph_pthread_mutex_lock(pthread_mutex_t *mptr);
void ph_pthread_mutex_unlock(pthread_mutex_t *mptr);
void ph_pthread_cond_init(pthread_cond_t *cptr, const pthread_condattr_t *attr);
void ph_pthread_cond_destroy(pthread_cond_t *cptr);
void ph_pthread_cond_broadcast(pthread_cond_t *cptr);
void ph_pthread_cond_signal(pthread_cond_t *cptr);
void ph_pthread_cond_wait(pthread_cond_t *cptr, pthread_mutex_t *mptr);
void ph_pthread_cond_timedwait(pthread_cond_t *cptr, pthread_mutex_t *mptr, const struct timespec *tsptr);
void ph_pthread_once(pthread_once_t *ptr, void (*func)(void));
void ph_pthread_key_create(pthread_key_t *key, void (*func)(void *));
void ph_pthread_setspecific(pthread_key_t key, const void *value);
void ph_pthread_attr_init(pthread_attr_t *attr);
void ph_pthread_attr_destroy(pthread_attr_t *attr);
void ph_pthread_attr_setdetachstate(pthread_attr_t *attr, int detachstate);
void ph_pthread_attr_setscope(pthread_attr_t *attr, int contentionscope);
void ph_pthread_sigmask(int how, const sigset_t *set, sigset_t *oset);
#else
void ph_begin_thread(HANDLE * tid, void * (*func)(void *), void *arg);
void ph_pthread_wait(pthread_t tid);
void ph_initialize_critical_section(CRITICAL_SECTION * mptr);
void ph_enter_critical_section(CRITICAL_SECTION * mp);
void ph_leave_critical_section(CRITICAL_SECTION * mp);
void ph_delete_critical_section(CRITICAL_SECTION * mp);
void ph_cond_init(ph_cond_t *cptr);
void ph_cond_destroy(ph_cond_t *cptr);
void ph_cond_broadcast(ph_cond_t *cptr);
//void ph_pthread_cond_signal(pthread_cond_t *cptr);
void ph_cond_wait(ph_cond_t *cptr, CRITICAL_SECTION *mptr);
#endif


/* Following functions are defined in wrap_string.c */
void * ph_mdup(const void * s, size_t n);
void ph_mzero(void *s, size_t n);
void ph_mmove(void *s1, const void *s2, size_t n);
size_t ph_strlen(const char *s);
int ph_strcmp(const char *s1, const char *s2);
int ph_strncmp(const char *s1, const char *s2, size_t n);
int ph_streq(const char *s1, const char *s2);
char * ph_strncpy(char *s1, const char *s2, size_t n);
char * ph_strdup(const char *str);
int ph_vsnprintf(char *s, size_t n, const char *format,  va_list ap);
int ph_snprintf(char *s, size_t  n, const char *fmt, ...);



/* Following functions are defined in list.c */
ph_list_t ph_list_new(int is_sync);
void ph_list_dispose(ph_list_t list, void (*func)(void *));
int ph_list_is_empty(ph_list_t list);
int ph_list_is_empty_opt(ph_list_t list, int is_sync);
int ph_list_get_nelm(ph_list_t list);
void ph_list_insert(ph_list_t list, void * val);
void ph_list_delete(ph_list_t list, void * val, void (*func)(void *));
void ph_list_delete_opt(ph_list_t list, void * val, void (*func)(void *), int is_sync);
void ph_list_enqueue(ph_list_t list, void * val);
void * ph_list_dequeue(ph_list_t list);
void * ph_list_dequeue_opt(ph_list_t list, int is_block, int (*func)(void *, void *, void *), void * arg1, void * arg2);
void * ph_list_find(ph_list_t list, int (*func)(void *, void *), void * arg);
void ph_list_map(ph_list_t list, void (*func)(void *, void *), void * arg);
void ph_list_sort(ph_list_t p, int (*func)(void *, void *));
void ph_list_swap(ph_list_t p,  ph_list_t q, int is_sync);


#define ph_min(a,b)	((a) < (b) ? (a) : (b))
#define ph_max(a,b)	((a) > (b) ? (a) : (b))



/* Following functions enables us to define ***_list structure easily.  */

#define PH_DEFINE_LIST_STRUCTURE(name) \
struct name ## _cell { \
  name ## _t		self; \
  struct name ## _cell 	* next; \
}; \
\
typedef struct name ## _cell	* name ## _cell_t; \
\
struct name ## _list { \
  name ## _cell_t	head; \
  name ## _cell_t	tail; \
  int			nelm; \
  int			is_sync; \
  pthread_mutex_t	mp; \
  pthread_cond_t	i_cond; \
  pthread_cond_t	d_cond; \
}; \
\
typedef struct name ## _list	* name ## _list_t; 


#define PH_DEFINE_LIST_FUNC_PROTOTYPE(name) \
name ## _list_t name ## _list_new        (int is_sync); \
void            name ## _list_dispose    (name ## _list_t list); \
int             name ## _list_is_empty   (name ## _list_t list); \
int             name ## _list_is_empty_opt(name ## _list_t list, int is_sync); \
int             name ## _list_get_nelm   (name ## _list_t list); \
void            name ## _list_insert     (name ## _list_t list, name ## _t val); \
void            name ## _list_delete     (name ## _list_t list, name ## _t val); \
void            name ## _list_delete_opt (name ## _list_t list, name ## _t val, int is_sync); \
void            name ## _list_enqueue    (name ## _list_t list, name ## _t val); \
void            name ## _list_enqueue_opt(name ## _list_t list, name ## _t val, int is_sync); \
name ## _t      name ## _list_dequeue    (name ## _list_t list); \
name ## _t      name ## _list_dequeue_opt(name ## _list_t list, int is_block, int (*func)(name ## _t, void *, void *), void * arg1, void * arg2); \
name ## _t      name ## _list_find(name ## _list_t list, int (*func)(name ## _t, void *), void * arg); \
void            name ## _list_map        (name ## _list_t list, void (*func)(name ## _t, void *), void * arg); \
void	        name ## _list_sort       (name ## _list_t list, int (*func)(name ## _t, name ## _t)); \
void            name ## _list_swap       (name ## _list_t p, name ## _list_t q, int is_sync);


#define PH_DEFINE_LIST_FUNC_BODY(name) \
name ## _list_t name ## _list_new(int is_sync){ \
  return (name ##_list_t)ph_list_new(is_sync); \
} \
\
void name ## _list_dispose(name ## _list_t list) { \
  ph_list_dispose((ph_list_t)list, (ph_func1) &name ## _dispose); \
} \
\
int name ## _list_is_empty(name ## _list_t list) { \
  return ph_list_is_empty((ph_list_t)list); \
} \
\
int name ## _list_is_empty_opt(name ## _list_t list, int is_sync) { \
  return ph_list_is_empty_opt((ph_list_t)list, is_sync); \
} \
\
int             name ## _list_get_nelm  (name ## _list_t list) { \
  return ph_list_get_nelm((ph_list_t)list); \
} \
\
void name ## _list_insert(name ## _list_t list, name ## _t val) { \
  ph_list_insert((ph_list_t)list, (void *)val); \
} \
\
void name ## _list_delete(name ## _list_t list, name ## _t val) { \
  ph_list_delete((ph_list_t)list, (void *)val, (ph_func1)&name ## _dispose); \
}\
\
void name ## _list_delete_opt(name ## _list_t list, name ## _t val, int is_sync){ \
  ph_list_delete_opt((ph_list_t)list, (void *)val, (ph_func1)&name ## _dispose, is_sync); \
} \
\
void name ## _list_enqueue(name ## _list_t list, name ## _t val) { \
  ph_list_enqueue((ph_list_t)list, (void *)val); \
} \
\
name ## _t name ## _list_dequeue(name ## _list_t list) { \
  return (name ## _t)ph_list_dequeue((ph_list_t)list); \
} \
\
name ## _t name ## _list_dequeue_opt(name ## _list_t list, int is_block, int (*func)(name ## _t, void *, void *), void * arg1, void * arg2) { \
  return (name ## _t)ph_list_dequeue_opt((ph_list_t)list, is_block, (ph_func4)func, arg1, arg2); \
} \
\
name ## _t name ## _list_find(name ## _list_t list, int (*func)(name ## _t, void *), void * arg) { \
  return (name ## _t)ph_list_find((ph_list_t)list, (ph_func2)func, arg); \
} \
\
void name ## _list_map(name ## _list_t list, void (*func)(name ## _t, void *), void * arg) { \
  ph_list_map((ph_list_t)list, (ph_func3)func, arg); \
} \
void name ## _list_sort(name ## _list_t list, int (*func)(name ## _t, name ## _t)) { \
  ph_list_sort((ph_list_t)list, (ph_func2)func); \
} \
\
void name ## _list_swap(name ## _list_t p, name ## _list_t q, int is_sync) {\
  ph_list_swap((ph_list_t)p, (ph_list_t)q, is_sync); \
} \


#endif


  



