/**************************************
 * $Author: kaneda $
 * $Date: 2003/02/12 12:45:20 $
 **************************************/

/* user.c */

#include "ph_pt2pt.h"

void ph_pt2pt_initialize(ph_pt2pt_t pt2pt) {
  ph_pt2pt_run(pt2pt);
}

void ph_pt2pt_assume_vp_set(ph_pt2pt_t pt2pt, ph_vp_set_t vp_set) {
  ph_vp_set_sync_assume(pt2pt->vp_set, vp_set);
  ph_routing_event_list_enqueue(pt2pt->routing->event_list, ph_routing_event_new(assume_vps, NULL, ph_routing_entry_list_new2(vp_set)));
}

ph_vp_set_t ph_pt2pt_get_assumed_vps(ph_pt2pt_t pt2pt) {
  return ph_vp_set_sync_dup(pt2pt->vp_set);
}


void ph_pt2pt_release_vp_set(ph_pt2pt_t pt2pt, ph_vp_set_t vp_set) {
  ph_vp_set_sync_release(pt2pt->vp_set, vp_set);
  ph_routing_event_list_enqueue(pt2pt->routing->event_list, ph_routing_event_new(release_vps, NULL, ph_routing_entry_list_new2(vp_set)));
}


void ph_pt2pt_add_contact_point(ph_pt2pt_t pt2pt, char * addr, char * service, ph_addr_type_t addr_type, char * user) {
  char * host;
  
  assert(addr != NULL);
  assert(service != NULL);

  if ((host = ph_addr_get_host(addr, atoi(service), pt2pt->listen_port)) == NULL) {
    return;
  }

  ph_addr_list_insert(pt2pt->addr_list, ph_addr_new(host, service, addr_type, user));  
  ph_free(host);
}


void ph_pt2pt_send_user_msg(ph_pt2pt_t pt2pt, ph_vp_t dest, void * buf, size_t len, int tag) {
  ph_pt2pt_handle_msg(pt2pt, ph_msg_new_user(dest, (buf != NULL) ? ph_mdup(buf, len) : NULL, len, tag));
}


ph_msg_t ph_pt2pt_recv_user_msg(ph_pt2pt_t pt2pt, int tag, int is_block) {
  ph_msg_t msg;
  msg = ph_msg_list_dequeue_local_user_msg(pt2pt->msg_list, pt2pt->vp_set, tag, is_block);
  return msg;
}

void ph_pt2pt_invalidate_routing(ph_pt2pt_t pt2pt) {
  ph_pt2pt_set_is_finalizing(pt2pt, 1);
  ph_routing_event_list_enqueue(pt2pt->routing->event_list, ph_routing_event_new(finalize, NULL, NULL));
}

void ph_pt2pt_finalize(ph_pt2pt_t pt2pt) {
  ph_pt2pt_set_is_disposed(pt2pt, 1);
  ph_pt2pt_stop(pt2pt);
}


int ph_pt2pt_messages_is_empty(ph_pt2pt_t pt2pt) {
  return ph_msg_list_is_empty_user_msg(pt2pt->msg_list);
}


void ph_pt2pt_load_messages(ph_pt2pt_t pt2pt, char * filename) {
  assert(filename != NULL);
  ph_msg_list_load(pt2pt->msg_list, filename);
}


void ph_pt2pt_store_messages(ph_pt2pt_t pt2pt, char * filename) {
  assert(filename != NULL);
  ph_msg_list_store(pt2pt->msg_list, filename);  
}


ph_vp_t ph_pt2pt_get_temp_vp() {
  ph_vp_t  vp;

#ifndef __PH_WIN32__
  // lrand48 gets a random number from interval [0, 2**31].

  vp = ((ph_vp_t)(lrand48()) << 31) + (ph_vp_t)(lrand48());
 
  if (vp >= PHOENIX_VP_LIMIT) {
    vp = PHOENIX_MAX_VP;
  }
  vp = vp + PHOENIX_VP_LIMIT;

  
  assert((PHOENIX_MAX_VP <= vp) && (vp <= PH_MAX_VP));
#else 
  assert(0);
#endif
  return vp;
}


#ifdef PH_VIEWER
void ph_pt2pt_viewer_initialize(ph_pt2pt_t pt2pt, int viewer_update_interval, char * addr, char * service, ph_addr_type_t addr_type, char * user) {
  char * host;
  
  assert(addr != NULL);
  assert(service != NULL);

  if ((host = ph_addr_get_host(addr, atoi(service), pt2pt->listen_port)) == NULL) {
    return;
  }

  assert(pt2pt->viewer_addr == NULL);
  ph_pthread_mutex_lock(&pt2pt->mp);
  pt2pt->viewer_addr = ph_addr_new(host, service, addr_type, user);
  ph_pthread_mutex_unlock(&pt2pt->mp);
  ph_pt2pt_viewer_run(pt2pt, viewer_update_interval);
}


void ph_pt2pt_viewer_put_comment(ph_pt2pt_t pt2pt, void * body, size_t len) {
  ph_pthread_mutex_lock(&pt2pt->mp);
  ph_mzero(pt2pt->viewer_comment,100);
  ph_mmove(pt2pt->viewer_comment, body, ph_min(ph_strlen(body), 100));
  ph_pthread_mutex_unlock(&pt2pt->mp);
}

#endif


