/**************************************
 * $Author: kaneda $
 * $Date: 2002/10/16 03:00:04 $
 **************************************/

/* wrap_pthread.c */


#include "ph_util.h"

#ifndef __PH_WIN32__

void ph_pthread_create(pthread_t *tid, const pthread_attr_t *attr, void * (*func)(void *), void *arg) {
  int n;
  
  if ((n = pthread_create(tid, attr, func, arg)) == 0) {
    return;
  }
  errno = n;
  ph_err_sys("pthread_create error");
}

void ph_pthread_join(pthread_t tid, void **status) {
  int n;

  if ((n = pthread_join(tid, status)) == 0) {
    return;
  }
  errno = n;
  ph_err_sys("pthread_join error");
}

void ph_pthread_detach(pthread_t tid) {
  int n;
 
  if ((n = pthread_detach(tid)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_detach error");
}

void ph_pthread_mutexattr_init(pthread_mutexattr_t *attr) {
  int n;

  if ((n = pthread_mutexattr_init(attr)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_mutexattr_init error");
}

void ph_pthread_mutex_init(pthread_mutex_t *mptr, pthread_mutexattr_t *attr) {
  int n;

  if ((n = pthread_mutex_init(mptr, attr)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_mutex_init error");
}


void ph_pthread_mutex_destroy(pthread_mutex_t *mptr) {
  int n;
  
  if ((n = pthread_mutex_destroy(mptr)) == 0) {
    return;
  }
  
  errno = n;
  ph_err_sys("pthread_mutex_destroy error");
}


void ph_pthread_mutex_lock(pthread_mutex_t *mptr) {
  int n;

  if ((n = pthread_mutex_lock(mptr)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_mutex_lock error");
}


void ph_pthread_mutex_unlock(pthread_mutex_t *mptr) {
  int n;
  
  if ((n = pthread_mutex_unlock(mptr)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_mutex_unlock error");
}


void ph_pthread_cond_init(pthread_cond_t *cptr, const pthread_condattr_t *attr) {
  int n;

  if ((n = pthread_cond_init(cptr, attr)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_cond_init error");
}


void ph_pthread_cond_destroy(pthread_cond_t *cptr) {
  int n;
  
  if ((n = pthread_cond_destroy(cptr)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_cond_destroy error");
}


void ph_pthread_cond_broadcast(pthread_cond_t *cptr) {
  int n;

  if ((n = pthread_cond_broadcast(cptr)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_cond_broadcast error");
}
 
void ph_pthread_cond_signal(pthread_cond_t *cptr) {
  int n;

  if ((n = pthread_cond_signal(cptr)) == 0) {
    return;
  }
  
  errno = n;
  ph_err_sys("pthread_cond_signal error");
}

void ph_pthread_cond_wait(pthread_cond_t *cptr, pthread_mutex_t *mptr) {
  int n;

  if ((n = pthread_cond_wait(cptr, mptr)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_cond_wait error");
}

void ph_pthread_cond_timedwait(pthread_cond_t *cptr, pthread_mutex_t *mptr, const struct timespec *tsptr) {
  int n;

  if ((n = pthread_cond_timedwait(cptr, mptr, tsptr)) == 0) {
    return;
  }
  
  errno = n;
  ph_err_sys("pthread_cond_timedwait error");
}

void ph_pthread_once(pthread_once_t *ptr, void (*func)(void)) {
  int n;
  
  if ((n = pthread_once(ptr, func)) == 0) {
    return;
  }
	
  errno = n;
  ph_err_sys("pthread_once error");
}

void ph_pthread_key_create(pthread_key_t *key, void (*func)(void *)) {
  int n;

  if ((n = pthread_key_create(key, func)) == 0) {
    return;
  }
	
  errno = n;
  ph_err_sys("pthread_key_create error");
}

void ph_pthread_setspecific(pthread_key_t key, const void *value) {
  int n;

  if ((n = pthread_setspecific(key, value)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_setspecific error");
}

void ph_pthread_attr_init(pthread_attr_t *attr) {
  int n;

  if ((n = pthread_attr_init(attr)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_attr_init error");
}


void ph_pthread_attr_destroy(pthread_attr_t *attr) {
  int n;

  if ((n = pthread_attr_destroy(attr)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_attr_destroy error");
}

void ph_pthread_attr_setdetachstate(pthread_attr_t *attr, int detachstate) {
  int n;

  if ((n = pthread_attr_setdetachstate(attr, detachstate)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_attr_setdetachstate error");
}

void ph_pthread_attr_setscope(pthread_attr_t *attr, int contentionscope) {
  int n;
  if ((n = pthread_attr_setscope(attr, contentionscope)) == 0) {
    return;
  }

  errno = n;
  ph_err_sys("pthread_attr_setscope error");

}


void ph_pthread_sigmask(int how, const sigset_t *set, sigset_t *oset) {
  int n;
  
  if ((n = pthread_sigmask(how, set, oset)) == 0) {
    return;
  }
  
  errno = n;
  ph_err_sys("pthread_sigmask error");
}

#else


void ph_begin_thread(HANDLE * tid, void * (*func)(void *), void *arg) {
  if (((*tid) = (HANDLE)_beginthread((void (*)(void *))func, 0, arg)) > 0) {
    return;
  }
  ph_err_sys("_beginthread error");
}

void ph_pthread_wait(pthread_t tid) {
  WaitForSingleObject(tid, INFINITE);
  CloseHandle(tid);
}

void ph_initialize_critical_section(CRITICAL_SECTION * mptr) {
  InitializeCriticalSection(mptr);
}

void ph_enter_critical_section(CRITICAL_SECTION * mp) {
  EnterCriticalSection(mp);
}

void ph_leave_critical_section(CRITICAL_SECTION * mp) {
  LeaveCriticalSection(mp);
}

void ph_delete_critical_section(CRITICAL_SECTION * mp) {
  DeleteCriticalSection(mp);
}



static void ph_waiters_init(ph_cond_t * cptr) {
  cptr->front = 0;
  cptr->rear = PH_MAX_THREAD-1;
}

static void ph_waiters_destroy(ph_cond_t * cptr) {
}

static int ph_waiters_is_empty(ph_cond_t * cptr) {
  return ((cptr->rear + 1) % PH_MAX_THREAD == cptr->front);
}

static int ph_waiters_is_full(ph_cond_t * cptr) {
  return ((cptr->rear + 2) % PH_MAX_THREAD == cptr->front);
}

static void ph_waiters_enqueue(ph_cond_t * cptr, HANDLE p) {
  assert(ph_waiters_is_full(cptr) == 0);
  cptr->rear = (cptr->rear + 1) % PH_MAX_THREAD;
  cptr->waiters[cptr->rear] = p;
}

static HANDLE ph_waiters_dequeue(ph_cond_t * cptr) {
  HANDLE p;

  if (ph_waiters_is_empty(cptr)) {
    return NULL;
  }

  p = cptr->waiters[cptr->front];
  cptr->front = (cptr->front + 1) % PH_MAX_THREAD;
  return p;
}


void ph_cond_init(ph_cond_t * cptr) {
  InitializeCriticalSection(&cptr->mp);
  ph_waiters_init(cptr);
}

void ph_cond_destroy(ph_cond_t * cptr) {
  if (cptr == NULL) {
    ph_err_msg("ph_cond_destroy() : NULL input");
    return;
  }
  ph_delete_critical_section(&cptr->mp);
  ph_waiters_destroy(cptr);
}

void ph_cond_broadcast(ph_cond_t * cptr) {
  HANDLE p;

  while ((p = ph_waiters_dequeue(cptr->waiters)) != NULL) {
    ReleaseSemaphore(p, 1, NULL);    
  }
}

void ph_cond_wait(ph_cond_t * cptr, CRITICAL_SECTION *mptr) {
  HANDLE p;

  p = CreateSemaphore(NULL,  0, 1, NULL);
  ph_waiters_enqueue(cptr->waiters, p);
  ph_leave_critical_section(mptr);

  WaitForSingleObject(p, INFINITE);

  ph_enter_critical_section(mptr);
  CloseHandle((HANDLE)p);
}

void ph_cond_timedwait(ph_cond_t * cptr, CRITICAL_SECTION *mptr, DWORD msec) {
  HANDLE p;

  p = CreateSemaphore(NULL,  0, 1, NULL);
  ph_waiters_enqueue(cptr->waiters, p);
  ph_leave_critical_section(mptr);

  WaitForSingleObject(p, msec);

  ph_enter_critical_section(mptr);
  CloseHandle((HANDLE)p);
}

//void ph_pthread_cond_signal(HANDLE *cptr) {
//  
//}

#endif
