/**************************************
 * $Author: kaneda $
 * $Date: 2002/11/15 02:50:46 $
 **************************************/

/* wrap_sock.c */

#include "ph_util.h"


int ph_accept(int fd, struct sockaddr *sa, socklen_t *salenptr) {
  int n;

#ifndef __PH_WIN32__  
 again:
  if ((n = accept(fd, sa, salenptr)) < 0) {
    if (errno == ECONNABORTED) {
      goto again;
    } else {
      ph_err_sys("accept error");
    }
  }
#else
  if ((n = accept(fd, sa, salenptr)) < 0) {
    ph_err_sys("accept error");
  }
#endif
  return(n);
}

void ph_bind(int fd, const struct sockaddr *sa, socklen_t salen) {
  if (bind(fd, sa, salen) < 0) {
    ph_err_sys("bind error");
  }
}

void ph_connect(int fd, const struct sockaddr *sa, socklen_t salen) {
  if (connect(fd, sa, salen) < 0) {
    ph_err_sys("connect error");
  }
}

void ph_getpeername(int fd, struct sockaddr *sa, socklen_t *salenptr) {
  if (getpeername(fd, sa, salenptr) < 0) {
    ph_err_sys("getpeername error");
  }
}

void ph_getsockname(int fd, struct sockaddr *sa, socklen_t *salenptr) { 
  if (getsockname(fd, sa, salenptr) < 0) {
    ph_err_sys("getsockname error");
  }
}

void ph_getsockopt(int fd, int level, int optname, void *optval, socklen_t *optlenptr) {
  if (getsockopt(fd, level, optname, optval, optlenptr) < 0) {
    ph_err_sys("getsockopt error");
  }
}


void ph_listen(int fd, int backlog) {
  char*ptr;
  
  if ((ptr = getenv("LISTENQ")) != NULL) {
    backlog = atoi(ptr);
  }
  
  if (listen(fd, backlog) < 0) {
    ph_err_sys("listen error");
  }
}

/*
ssize_t ph_recv(int fd, void *ptr, size_t nbytes, int flags) {
  ssize_t n;

  if ((n = recv(fd, ptr, nbytes, flags)) < 0) {
    ph_err_sys("recv error");
  }

  return(n);
}


ssize_t ph_recvfrom(int fd, void *ptr, size_t nbytes, int flags, struct sockaddr *sa, socklen_t *salenptr) {
  ssize_t n;

  if ((n = recvfrom(fd, ptr, nbytes, flags, sa, salenptr)) < 0) {
    ph_err_sys("recvfrom error");
  }
  return(n);
}

ssize_t ph_recvmsg(int fd, struct msghdr *msg, int flags) {
  ssize_t n;

  if ((n = recvmsg(fd, msg, flags)) < 0) {
    ph_err_sys("recvmsg error");
  }
  return(n);
}
*/


int ph_select(int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout) {
  int n;

  if ((n = select(nfds, readfds, writefds, exceptfds, timeout)) < 0) {
    //ph_err_msg("select error");
  }

  return(n);		/* can return 0 on timeout */
}

/*
void ph_send(int fd, const void *ptr, size_t nbytes, int flags) {
  if (send(fd, ptr, nbytes, flags) != nbytes) {
    ph_err_sys("send error");
  }
}

void ph_sendto(int fd, const void *ptr, size_t nbytes, int flags, const struct sockaddr *sa, socklen_t salen) {
  if (sendto(fd, ptr, nbytes, flags, sa, salen) != nbytes) {
    ph_err_sys("sendto error");
  }
}

void ph_sendmsg(int fd, const struct msghdr *msg, int flags) {
  int		i;
  ssize_t	nbytes;

  nbytes = 0;	// must first figure out what return value should be
  for (i = 0; i < msg->msg_iovlen; i++) {
    nbytes += msg->msg_iov[i].iov_len;
    
    if (sendmsg(fd, msg, flags) != nbytes) {
      ph_err_sys("sendmsg error");
    }
  }
}
*/

void ph_setsockopt(int fd, int level, int optname, const void *optval, socklen_t optlen) {
  if (setsockopt(fd, level, optname, optval, optlen) < 0) {
    ph_err_sys("setsockopt error");
  }
}

void ph_shutdown(int fd, int how) {
  if (shutdown(fd, how) < 0) {
    ph_err_sys("shutdown error");
  }
}


int ph_socket(int family, int type, int protocol) {
  int n;

  if ((n = socket(family, type, protocol)) < 0) {
    ph_err_sys("socket error");
  }
  return(n);
}

#ifndef __PH_WIN32__
void ph_socketpair(int family, int type, int protocol, int *fd) {
  int n;

  if ((n = socketpair(family, type, protocol, fd)) < 0) {
    ph_err_sys("socketpair error");
  }
}
#endif


char * ph_sock_ntop(const struct sockaddr *sa, socklen_t salen) {
  char * s;
  struct sockaddr_in	*sin = (struct sockaddr_in *) sa;
  
  if ((s = inet_ntoa(sin->sin_addr)) == NULL) {
    ph_err_sys("inet_ntoa error");
    return NULL;
  }
  
  return s;
}








