/**************************************
 * $Author: kaneda $
 * $Date: 2002/12/18 07:55:55 $
 **************************************/

/* wrap_string.c */

#include "ph_util.h"


void * ph_mdup(const void * s, size_t n) {
  void * t;
  
  if ((s == NULL) || (n <= 0)) {
    ph_err_msg("ph_mdup()\t: wrong input");
    return NULL;
  }

  t = ph_malloc(n);
  memmove(t, s, n);

  return t;
}


void ph_mzero(void *s, size_t n) {
  assert(s != NULL);
  assert(n > 0);
  memset(s, 0, n);
}


void ph_mmove(void *s1, const void *s2, size_t n) {
  assert(s1 != NULL);
  assert(s2 != NULL);
  assert(n > 0);
  memmove(s1, s2, n);
}


size_t ph_strlen(const char *s) {
  assert(s != NULL);
  return strlen(s);
}


int ph_strcmp(const char *s1, const char *s2) {
  assert(s1 != NULL);
  assert(s2 != NULL);
  return strcmp(s1, s2);
}


int ph_strncmp(const char *s1, const char *s2, size_t n) {
  assert(s1 != NULL);
  assert(s2 != NULL);
  return strncmp(s1, s2, n);
}


int ph_streq(const char *s1, const char *s2) {
  assert(s1 != NULL);
  assert(s2 != NULL);
  return (ph_strcmp(s1, s2) == 0);
}

char * ph_strncpy(char *s1, const char *s2, size_t n) {
  char * ptr;  
  
  assert(s1 != NULL);
  assert(s2 != NULL);
  assert(n > 0);

  if ((ptr = strncpy(s1, s2, n)) == NULL) {
    ph_err_sys("strncpy error");
  }
  
  return (ptr);
}

char * ph_strdup(const char * s) {
  assert(s != NULL);
  return (strdup(s));
}

int ph_vsnprintf(char *s, size_t n, const char *format,  va_list ap) {
  int m;
  
  if ((m = vsnprintf(s, n, format, ap)) >= 0) {
    return m;
  }
  
  errno = m;
  ph_err_sys("ph_vsnprintf error");
  return m;
}


int ph_snprintf(char *s, size_t  n, const char *fmt, ...) {
  va_list ap;
  int m;

  assert(s != NULL);
  assert(n > 0);
  
  va_start(ap, fmt);
  m = ph_vsnprintf(s, n, fmt, ap);
  va_end(ap);

  return m;
}











