#include "viewer.h"


viewer_t viewer_new(int listen_port) {
  viewer_t viewer;
   
  viewer = (viewer_t)ph_malloc(sizeof(struct viewer));

  {
    int lp, lps[2];
    lps[0] = lps[1] = listen_port;
    viewer->listen_conn = ph_conn_listen(lps, &lp);
    assert(viewer->listen_conn != NULL);
  }

  viewer->vinfo_list = vinfo_list_new(0);
  initialize_X(viewer);


  /* print prompt */
  printf("\n> ");

  return viewer;
}


void viewer_dispose(viewer_t viewer) {
  if (viewer == NULL) {
    ph_err_msg("viewer_dispose() : NULL input");
    return;
  }
  ph_conn_dispose(viewer->listen_conn);
  vinfo_list_dispose(viewer->vinfo_list);
  finalize_X(viewer);
  ph_free(viewer);
}


static void initialize_select(fd_set * r_fdset_p, fd_set * e_fdset_p) {
  FD_ZERO(r_fdset_p);
  FD_ZERO(e_fdset_p);
}

static void check_io(viewer_t viewer, fd_set * r_fdset_p, fd_set * e_fdset_p) {
  vinfo_cell_t	p;

  initialize_select(r_fdset_p, e_fdset_p);

  FD_SET(0, r_fdset_p);
  (*viewer->listen_conn->fd_set_func_p)(viewer->listen_conn->attr, r_fdset_p, PH_FD_TYPE_READ);
  (*viewer->listen_conn->fd_set_func_p)(viewer->listen_conn->attr, e_fdset_p, PH_FD_TYPE_ERROR);
  
  for (p = viewer->vinfo_list->head; p != NULL; p = p->next) {
    (*p->self->conn->fd_set_func_p)(p->self->conn->attr, r_fdset_p, PH_FD_TYPE_READ);
    (*p->self->conn->fd_set_func_p)(p->self->conn->attr, e_fdset_p, PH_FD_TYPE_ERROR);
  }
  
  ph_select(PH_MAX_FD, r_fdset_p, NULL, e_fdset_p, NULL);
}

vinfo_t vinfo_list_get_nth(vinfo_list_t vinfo_list, int no) {
  int i;
  vinfo_cell_t p;
  
  assert(vinfo_list != NULL);

  for (p = vinfo_list->head, i = 0; p != NULL; p = p->next, i++) {
    if (i == no) {
      return p->self;
    }
  }
  return NULL;
}


static void handle_user_event(viewer_t viewer, fd_set * r_fdset_p) {
  int no;
  char s[8192];
  vinfo_t vinfo;

  if (FD_ISSET(0, r_fdset_p) == 0) {
    return;
  }

  scanf("%d", &no);

  if ((vinfo = vinfo_list_get_nth(viewer->vinfo_list, no)) == NULL) {
    printf("No. %d does not exist.\n> ", no);
    return;
  }

  /* print node infomation */
  printf("[%s:%s]\n", vinfo->host, vinfo->port);

  ph_mzero(s, 8192);
  ph_vp_set_vps2str_for_print(vinfo->vp_set, s);
  printf("vps=%s\n", s);

  {
    ph_routing_entry_cell_t p;

    printf("---routing---\ndest\tconn\tmetric\n");
    for (p = vinfo->entry_list->head; p != NULL; p = p->next) {
      printf("[" PH_LLD "," PH_LLD "] %d %d\n", p->self->dest->lower, p->self->dest->upper, (int)p->self->conn, p->self->metric);
    }
  }

  {
    ph_msg_cell_t p;

    printf("---msg---\ndest\tconn\ttype\ttag\tlen\n");
    for (p = vinfo->msg_list->head; p != NULL; p = p->next) {
      printf(PH_LLD " %d %s %d\n", 
	     p->self->dest, 
	     (int)p->self->to_conn, 
	     ((p->self->type == meta) ? "meta" : "user"),
	     p->self->dest,
	     p->self->len,
	     p->self->tag);
    }
  }


  printf("comment=\"%s\"\n", vinfo->comment);

  printf("\n> ");
}

static void handle_accepting(viewer_t viewer, fd_set * r_fdset_p) {
  if ((*viewer->listen_conn->fd_isset_func_p)(viewer->listen_conn->attr, r_fdset_p, PH_FD_TYPE_READ) == 0) {
    return;
  }
  vinfo_list_insert(viewer->vinfo_list, vinfo_new(ph_conn_accept(viewer->listen_conn)));
}

static void handle_opened(viewer_t viewer, vinfo_cell_t vinfo_cell, fd_set * r_fdset_p) {
  ph_msg_t msg;
  ph_conn_t conn;
  
  conn = vinfo_cell->self->conn;
  if ((*conn->fd_isset_func_p)(conn->attr, r_fdset_p, PH_FD_TYPE_READ) == 0) {
    return;
  }

  if ((msg = ph_msg_recv(conn)) == NULL) {
    ph_conn_set_state(conn, closed);
    return;
  }

  vinfo_cell->self->conn = NULL;
  vinfo_dispose(vinfo_cell->self);
  
  vinfo_cell->self = vinfo_new2(conn, msg->body);
  ph_msg_dispose(msg);
  return;
}

static int handle_error(viewer_t viewer, vinfo_t vinfo, fd_set * e_fdset_p) {
  if ((*vinfo->conn->fd_isset_func_p)(vinfo->conn->attr, e_fdset_p, PH_FD_TYPE_ERROR) == 0) {
    return;
  }
  ph_conn_set_state(vinfo->conn, closed);
}

static void handle_io(viewer_t viewer, fd_set * r_fdset_p, fd_set * e_fdset_p) {
  vinfo_cell_t p;
  
  handle_user_event(viewer, r_fdset_p);

  handle_accepting(viewer, r_fdset_p);

  p = viewer->vinfo_list->head;
  while (p != NULL) {
    vinfo_cell_t next = p->next;

    handle_opened(viewer, p, r_fdset_p);
    handle_error(viewer, p->self, e_fdset_p);
    if (ph_conn_get_state(p->self->conn) == closed) {
      vinfo_list_delete(viewer->vinfo_list, p->self);
    }
    p = next;
  }
}

static void main_loop(viewer_t viewer) {
  while (1) {
    fd_set r_fdset, e_fdset;
    
    check_io(viewer, &r_fdset, &e_fdset);
    handle_io(viewer, &r_fdset, &e_fdset);
    redraw(viewer);
  }
}


/* receive messages and display information */
int main(int argn, char ** argv, char ** envp) {
  viewer_t viewer;

  if (argn != 2) {
    printf("Usage: %s port\n", argv[0]);
    return 1;
  }

  viewer = viewer_new(atoi(argv[1]));
  main_loop(viewer);
  viewer_dispose(viewer);
}




