#include "viewer.h"


vinfo_t vinfo_new(ph_conn_t conn) {
  vinfo_t vinfo;

  vinfo = (vinfo_t)ph_malloc(sizeof(struct vinfo));
  vinfo->conn = conn;
  vinfo->vp_set = ph_vp_set_make_empty();
  vinfo->reachable_vp_set = ph_vp_set_make_empty();
  vinfo->entry_list = ph_routing_entry_list_new(0);
  vinfo->msg_list = ph_msg_list_new(0);
  ph_mzero(vinfo->host, 100);
  ph_mzero(vinfo->port, 100);
  ph_mzero(vinfo->comment, 100);
  return vinfo;
}


void vinfo_dispose(vinfo_t vinfo) { 
  if (vinfo == NULL) {
    ph_err_msg("vinfo_dispose() : NULL input");
    return;
  }

  if (vinfo->conn != NULL) {
    ph_conn_dispose(vinfo->conn);
  }

  ph_vp_set_dispose(vinfo->vp_set);
  ph_vp_set_dispose(vinfo->reachable_vp_set);
  ph_routing_entry_list_dispose(vinfo->entry_list);
  ph_msg_list_dispose(vinfo->msg_list);
  ph_free(vinfo);
}


vinfo_t vinfo_new2(ph_conn_t conn, char * body) {
  vinfo_t vinfo;
  int i, r_nelm, m_nelm;
  int PH_ENTRY_SIZE = 100;
  
  assert(body != NULL);
  
  vinfo = vinfo_new(conn);

  ph_mmove(vinfo->host, body, 100);
  ph_mmove(vinfo->port, body+100, 100);
  ph_mmove(vinfo->comment, body+200, 100);

  sscanf(body+300, "%d %d", &r_nelm, &m_nelm);

  for (i = 0; i < r_nelm; i++) {
    ph_vp_t lower, upper;
    int conn_no, metric;
    ph_addr_type_t addr_type;
    
    sscanf(body + 400 + i*PH_ENTRY_SIZE, 
	   PH_LLD " " PH_LLD " %d %d %d" , 
	   &lower, &upper, &conn_no, &metric, (int *)&addr_type);
    ph_routing_entry_list_enqueue(vinfo->entry_list, 
				  ph_routing_entry_new(lower, upper, (ph_conn_t)conn_no, metric, 0, 0, NULL, addr_type));

    if (metric == 0) {
      ph_vp_set_add_range(&vinfo->vp_set, lower, upper);
    }
    if (metric < PH_MAX_METRIC) {
      ph_vp_set_add_range(&vinfo->reachable_vp_set, lower, upper);
    }
  }

  for (i = 0; i < m_nelm; i++) {
    ph_vp_t dest;
    int conn_no, tag, type;
    size_t len;
    ph_msg_t msg;

    sscanf(body + 400 + (r_nelm+i)*PH_ENTRY_SIZE, 
	   "%d " PH_LLD " %d %d %d\n", 
	   &type, &dest, &len, &tag, &conn_no);
    msg = ph_msg_new(type, dest, NULL, len, tag);
    ph_msg_list_enqueue(vinfo->msg_list, msg);
    msg->to_conn = (ph_conn_t)conn_no;
  }
  
  return vinfo;
}


void vinfo_vinfo2str(vinfo_t vinfo, char * s, size_t len) {
  assert(s != NULL);
  ph_mzero(s, len);

  snprintf(s, 30, "[%s:%s]  vps=", vinfo->host, vinfo->port);
  ph_vp_set_vps2str_for_print(vinfo->vp_set, s);
  snprintf(s + strlen(s), 30, ", n_msgs=%d, reachable=",  ph_msg_list_get_nelm(vinfo->msg_list));
  ph_vp_set_vps2str_for_print(vinfo->reachable_vp_set, s);
}


int vinfo_compare(vinfo_t p, vinfo_t q) {
  assert(p != NULL);
  assert(q != NULL);
  return ((ph_strncmp(p->host, q->host, 100) > 0) ||
	  ((ph_strncmp(p->host, q->host, 100) == 0) &&
	   (ph_strncmp(p->port, q->port, 100) > 0)));
}



int vinfo_is_connect(vinfo_t p, vinfo_t q) {
  ph_routing_entry_cell_t r;

  assert(p != NULL);
  assert(q != NULL);

  for (r = p->entry_list->head; r != NULL; r = r->next) {
    ph_vp_set_t tmp;

    if (r->self->metric != 1) {
      continue;
    }

    tmp = ph_vp_set_make_range(r->self->dest->lower, r->self->dest->upper);
    if (ph_vp_set_is_exclusive(tmp, q->vp_set) == 0) {
      ph_vp_set_dispose(tmp);
      return (int)r->self->addr_type;
    }
    ph_vp_set_dispose(tmp);
  }

  return -1;
}


int vinfo_is_routable(vinfo_t p, vinfo_t q) {
  ph_routing_entry_cell_t r;

  assert(p != NULL);
  assert(q != NULL);

  for (r = p->entry_list->head; r != NULL; r = r->next) {
    ph_vp_set_t tmp;

    if (r->self->metric >= PH_MAX_METRIC) {
      continue;
    }

    tmp = ph_vp_set_make_range(r->self->dest->lower, r->self->dest->upper);
    if (ph_vp_set_is_exclusive(tmp, q->vp_set) == 0) {
      ph_vp_set_dispose(tmp);
      return 1;
    }
    ph_vp_set_dispose(tmp);
  }
  return 0;
}



PH_DEFINE_LIST_FUNC_BODY(vinfo);






