/**************************************
 * $Author: kaneda $
 * $Date: 2002/12/20 10:14:40 $
 **************************************/

/* conn.h */


#ifndef __CONN_H__
#define __CONN_H__

#include "util.h"
#include "task.h"


enum conn_type {
  LISTENING,
  SHELL,
  PHOENIX,
  TASK_IN,
  TASK_OUT
}; 

typedef enum conn_type		conn_type_t;


enum conn_state {
  OPEN, CLOSING, CLOSED
};

typedef enum conn_state 	conn_state_t;


struct conn {
  conn_type_t 		type;
  conn_state_t 		state;
  int 			fd;

  ph_vp_t		vp;
  task_id_t		id;
};

typedef struct conn		* conn_t;


DEFINE_LIST_STRUCTURE(conn)


conn_t conn_new(conn_type_t type, int fd, ph_vp_t vp, task_id_t id);
void conn_dispose(conn_t conn);
conn_t conn_listen(int listen_port);
conn_t conn_accept(conn_t listen_conn, ph_vp_t vp);
ssize_t conn_send(conn_t conn, void * buf, size_t len);
ssize_t conn_recv(conn_t conn, void * buf, size_t len);
void conn_close(conn_t conn);
void ph_conn_lock(ph_conn_t conn);
void ph_conn_unlock(ph_conn_t conn);
conn_state_t conn_get_state(conn_t conn);
void conn_set_state(conn_t conn, conn_state_t state);
void conn_try_set_state(conn_t conn, conn_state_t state);



DEFINE_LIST_FUNC_PROTOTYPE(conn)


void conn_list_print(conn_list_t conn_list);
conn_cell_t conn_list_init_loop(conn_list_t conn_list);
conn_cell_t conn_list_iterate(conn_list_t conn_list, conn_cell_t p);
conn_t conn_list_find_same_dest(conn_list_t conn_list, ph_vp_t vp, task_id_t id);


#endif












