/**************************************
 * $Author: kaneda $
 * $Date: 2002/12/20 10:14:41 $
 **************************************/

/* wrap_pthread.c */


#include "util.h"


void Pthread_create(pthread_t *tid, const pthread_attr_t *attr, void * (*func)(void *), void *arg) {
  int n;
  
  if ((n = pthread_create(tid, attr, func, arg)) == 0) {
    return;
  }
  errno = n;
  err_sys("pthread_create error");
}

void Pthread_join(pthread_t tid, void **status) {
  int n;

  if ((n = pthread_join(tid, status)) == 0) {
    return;
  }
  errno = n;
  err_sys("pthread_join error");
}

void Pthread_detach(pthread_t tid) {
  int n;
 
  if ((n = pthread_detach(tid)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_detach error");
}

void Pthread_mutexattr_init(pthread_mutexattr_t *attr) {
  int n;

  if ((n = pthread_mutexattr_init(attr)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_mutexattr_init error");
}

void Pthread_mutex_init(pthread_mutex_t *mptr, pthread_mutexattr_t *attr) {
  int n;

  if ((n = pthread_mutex_init(mptr, attr)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_mutex_init error");
}


void Pthread_mutex_destroy(pthread_mutex_t *mptr) {
  int n;
  
  if ((n = pthread_mutex_destroy(mptr)) == 0) {
    return;
  }
  
  errno = n;
  err_sys("pthread_mutex_destroy error");
}


void Pthread_mutex_lock(pthread_mutex_t *mptr) {
  int n;

  if ((n = pthread_mutex_lock(mptr)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_mutex_lock error");
}


void Pthread_mutex_unlock(pthread_mutex_t *mptr) {
  int n;
  
  if ((n = pthread_mutex_unlock(mptr)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_mutex_unlock error");
}


void Pthread_cond_init(pthread_cond_t *cptr, const pthread_condattr_t *attr) {
  int n;

  if ((n = pthread_cond_init(cptr, attr)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_cond_init error");
}


void Pthread_cond_destroy(pthread_cond_t *cptr) {
  int n;
  
  if ((n = pthread_cond_destroy(cptr)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_cond_destroy error");
}


void Pthread_cond_broadcast(pthread_cond_t *cptr) {
  int n;

  if ((n = pthread_cond_broadcast(cptr)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_cond_broadcast error");
}
 
void Pthread_cond_signal(pthread_cond_t *cptr) {
  int n;

  if ((n = pthread_cond_signal(cptr)) == 0) {
    return;
  }
  
  errno = n;
  err_sys("pthread_cond_signal error");
}

void Pthread_cond_wait(pthread_cond_t *cptr, pthread_mutex_t *mptr) {
  int n;

  if ((n = pthread_cond_wait(cptr, mptr)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_cond_wait error");
}

void Pthread_cond_timedwait(pthread_cond_t *cptr, pthread_mutex_t *mptr, const struct timespec *tsptr) {
  int n;

  if ((n = pthread_cond_timedwait(cptr, mptr, tsptr)) == 0) {
    return;
  }
  
  errno = n;
  err_sys("pthread_cond_timedwait error");
}

void Pthread_once(pthread_once_t *ptr, void (*func)(void)) {
  int n;
  
  if ((n = pthread_once(ptr, func)) == 0) {
    return;
  }
	
  errno = n;
  err_sys("pthread_once error");
}

void Pthread_key_create(pthread_key_t *key, void (*func)(void *)) {
  int n;

  if ((n = pthread_key_create(key, func)) == 0) {
    return;
  }
	
  errno = n;
  err_sys("pthread_key_create error");
}

void Pthread_setspecific(pthread_key_t key, const void *value) {
  int n;

  if ((n = pthread_setspecific(key, value)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_setspecific error");
}

void Pthread_attr_init(pthread_attr_t *attr) {
  int n;

  if ((n = pthread_attr_init(attr)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_attr_init error");
}


void Pthread_attr_destroy(pthread_attr_t *attr) {
  int n;

  if ((n = pthread_attr_destroy(attr)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_attr_destroy error");
}

void Pthread_attr_setdetachstate(pthread_attr_t *attr, int detachstate) {
  int n;

  if ((n = pthread_attr_setdetachstate(attr, detachstate)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_attr_setdetachstate error");
}

void Pthread_attr_setscope(pthread_attr_t *attr, int contentionscope) {
  int n;
  if ((n = pthread_attr_setscope(attr, contentionscope)) == 0) {
    return;
  }

  errno = n;
  err_sys("pthread_attr_setscope error");

}


void Pthread_sigmask(int how, const sigset_t *set, sigset_t *oset) {
  int n;
  
  if ((n = pthread_sigmask(how, set, oset)) == 0) {
    return;
  }
  
  errno = n;
  err_sys("pthread_sigmask error");
}
