/**************************************
 * $Author: kaneda $
 * $Date: 2002/12/20 10:14:41 $
 **************************************/

/* wrap_sock.c */

#include "util.h"


int Accept(int fd, struct sockaddr *sa, socklen_t *salenptr) {
  int n;

 again:
  if ((n = accept(fd, sa, salenptr)) < 0) {
    if (errno == ECONNABORTED) {
      goto again;
    } else {
      err_sys("accept error");
    }
  }
  return(n);
}

void Bind(int fd, const struct sockaddr *sa, socklen_t salen) {
  if (bind(fd, sa, salen) < 0) {
    err_sys("bind error");
  }
}

void Connect(int fd, const struct sockaddr *sa, socklen_t salen) {
  if (connect(fd, sa, salen) < 0) {
    err_sys("connect error");
  }
}

void Getpeername(int fd, struct sockaddr *sa, socklen_t *salenptr) {
  if (getpeername(fd, sa, salenptr) < 0) {
    err_sys("getpeername error");
  }
}

void Getsockname(int fd, struct sockaddr *sa, socklen_t *salenptr) { 
  if (getsockname(fd, sa, salenptr) < 0) {
    err_sys("getsockname error");
  }
}

void Getsockopt(int fd, int level, int optname, void *optval, socklen_t *optlenptr) {
  if (getsockopt(fd, level, optname, optval, optlenptr) < 0) {
    err_sys("getsockopt error");
  }
}


void Listen(int fd, int backlog) {
  char*ptr;
  
  if ((ptr = getenv("LISTENQ")) != NULL) {
    backlog = atoi(ptr);
  }
  
  if (listen(fd, backlog) < 0) {
    err_sys("listen error");
  }
}


int Select(int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout) {
  int n;

  if ((n = select(nfds, readfds, writefds, exceptfds, timeout)) < 0) {
  }

  return(n);		/* can return 0 on timeout */
}


void Setsockopt(int fd, int level, int optname, const void *optval, socklen_t optlen) {
  if (setsockopt(fd, level, optname, optval, optlen) < 0) {
    err_sys("setsockopt error");
  }
}

void Shutdown(int fd, int how) {
  if (shutdown(fd, how) < 0) {
    err_sys("shutdown error");
  }
}


int Socket(int family, int type, int protocol) {
  int n;

  if ((n = socket(family, type, protocol)) < 0) {
    err_sys("socket error");
  }
  return(n);
}


void Socketpair(int family, int type, int protocol, int *fd) {
  int n;

  if ((n = socketpair(family, type, protocol, fd)) < 0) {
    err_sys("socketpair error");
  }
}


char * Sock_ntop(const struct sockaddr *sa, socklen_t salen) {
  char * s;
  struct sockaddr_in	*sin = (struct sockaddr_in *) sa;
  
  if ((s = inet_ntoa(sin->sin_addr)) == NULL) {
    err_sys("inet_ntoa error");
    return NULL;
  }
  
  return s;
}








