/**************************************
 * $Author: kaneda $
 * $Date: 2002/12/20 10:14:41 $
 **************************************/

/* wrap_string.c */

#include "util.h"


void * Mdup(const void * s, size_t n) {
  void * t;
  
  if ((s == NULL) || (n <= 0)) {
    err_msg("Mdup()\t: wrong input");
    return NULL;
  }

  t = Malloc(n);
  memmove(t, s, n);

  return t;
}


void Mzero(void *s, size_t n) {
  assert(s != NULL);
  assert(n > 0);
  memset(s, 0, n);
}


void Mmove(void *s1, const void *s2, size_t n) {
  assert(s1 != NULL);
  assert(s2 != NULL);
  assert(n > 0);
  memmove(s1, s2, n);
}


size_t Strlen(const char *s) {
  assert(s != NULL);
  return strlen(s);
}


int Strcmp(const char *s1, const char *s2) {
  assert(s1 != NULL);
  assert(s2 != NULL);
  return strcmp(s1, s2);
}


int Strncmp(const char *s1, const char *s2, size_t n) {
  assert(s1 != NULL);
  assert(s2 != NULL);
  return strncmp(s1, s2, n);
}


int Streq(const char *s1, const char *s2) {
  assert(s1 != NULL);
  assert(s2 != NULL);
  return (Strcmp(s1, s2) == 0);
}

char * Strncpy(char *s1, const char *s2, size_t n) {
  char * ptr;  
  
  assert(s1 != NULL);
  assert(s2 != NULL);
  assert(n > 0);

  if ((ptr = strncpy(s1, s2, n)) == NULL) {
    err_sys("strncpy error");
  }
  
  return (ptr);
}

char * Strdup(const char * s) {
  assert(s != NULL);
  return (strdup(s));
}

int Vsnprintf(char *s, size_t n, const char *format,  va_list ap) {
  int m;
  
  if ((m = vsnprintf(s, n, format, ap)) >= 0) {
    return m;
  }
  
  errno = m;
  err_sys("Vsnprintf error");
  return m;
}


int Snprintf(char *s, size_t  n, const char *fmt, ...) {
  va_list ap;
  int m;

  assert(s != NULL);
  assert(n > 0);
  
  va_start(ap, fmt);
  m = Vsnprintf(s, n, fmt, ap);
  va_end(ap);

  return m;
}











