/**************************************
 * $Author: kaneda $
 * $Date: 2002/12/20 10:14:42 $
 **************************************/

/* exec.c */

#include "vpgd.h"



static void spawn_task_exec(vpgd_t vpgd, task_t task) {
  int i_fds[2], o_fds[2];

  Pipe(i_fds);
  Pipe(o_fds);

  if ((task->pid = fork()) == 0) {
    /* redirect stdin to i_fds[0] */
    close(0);
    dup(i_fds[0]);
    close(i_fds[0]);

    /* redirect stdout to o_fds[1] */
    close(1);
    dup(o_fds[1]);
    close(o_fds[1]);

    /* close useless fds */
    close(i_fds[1]);
    close(o_fds[0]);

    assert(task->argv != NULL);
    assert(task->argv[0] != NULL);

    /* TODO: zsh $B$,(B zexecve() $B$G9T$C$F$$$k$h$&$JJQ49$,Ey$7$$(B (e.g., $B6uGr$r4^$`%U%!%$%kL>(B) */

    execvp(task->argv[0], task->argv);
    exit(1);
  }
  
  /* close useless fds */
  Close(i_fds[0]);
  Close(o_fds[1]);

  conn_list_insert(vpgd->conns, conn_new(TASK_IN, i_fds[1], vpgd_get_host_vp(vpgd), task->id));
  conn_list_insert(vpgd->conns, conn_new(TASK_OUT, o_fds[0], task->out_vp, task->out_id));

  // $B$9$G$K(B task $B$NF~NO08$KFO$$$F$$$k%a%C%;!<%8$,$"$C$?$i$=$l$r=hM}$9$k!%(B
  {
    msg_t msg;

    while ((msg = msg_list_dequeue_same_dest(vpgd->msgs, vpgd_get_host_vp(vpgd), task->id)) != NULL) {
      handle_msg(vpgd, msg, NULL);
    }
  }

  task->state = TASK_EXECUTING;
  task_list_insert(vpgd->tasks, task);  
}


static void spawn_task_redir_r(vpgd_t vpgd, task_t task) {
  int fd;

  if ((fd = open(task->argv[0], O_RDONLY | O_NOCTTY)) == -1) {
    return;
  }

  conn_list_insert(vpgd->conns, conn_new(TASK_OUT, fd, task->out_vp, task->out_id));
}


static void spawn_task_redir_w(vpgd_t vpgd, task_t task) {
  int fd;
 
  if ((fd = open(task->argv[0], O_WRONLY | O_CREAT | O_TRUNC | O_NOCTTY, 0666)) == -1) {
    return;
  }

  conn_list_insert(vpgd->conns, conn_new(TASK_IN, fd, vpgd_get_host_vp(vpgd), task->id));
  {
    msg_t msg;

    while ((msg = msg_list_dequeue_same_dest(vpgd->msgs, vpgd_get_host_vp(vpgd), task->id)) != NULL) {
      handle_msg(vpgd, msg, NULL);
    }
  }
}


void spawn_task(vpgd_t vpgd, task_t task) {
  debug_print4("spawn_task()\t\t: id=%lld, type=%d, argv[0]=%s, out_vp=%lld, oud_id=%lld\n", 
	       task->id, task->type, task->argv[0], task->out_vp, task->out_id);

  /* TODO: local$B4V$GDL?.$9$k%W%m%;%9$r$R$H$D$K$^$H$a$k(B */

  switch (task->type) {
  case EXEC:
    spawn_task_exec(vpgd, task);
    break;
  case REDIR_R:
    spawn_task_redir_r(vpgd, task);
    break;
  case REDIR_W:
    spawn_task_redir_w(vpgd, task);
    break;
  default:
    err_quit("spawn_task() : no matching");
    break;
  }
}


static void release_zombie_tasks_exec(vpgd_t vpgd, task_t task) {
  conn_t conn;

  if ((conn = conn_list_find_same_dest(vpgd->conns, vpgd_get_host_vp(vpgd), task->id)) != NULL) {
    assert(conn->type == TASK_IN);
    conn->state = (conn->state != CLOSED) ? CLOSING : CLOSED;
  }
}


void release_zombie_tasks(vpgd_t vpgd) {
  task_cell_t p;


  for (p = task_list_init_loop(vpgd->tasks); p != NULL; p = task_list_iterate(vpgd->tasks, p)) {
    int status;

    if (p->self->state == TASK_WAITING) {
      continue;
    }

    if (Waitpid(p->self->pid, &status, WNOHANG) == p->self->pid) {
      switch (p->self->type) {
      case EXEC:
	release_zombie_tasks_exec(vpgd, p->self);
	break;
      case REDIR_R:
      case REDIR_W:
	break;
      default:
	err_quit("release_zombie_tasks() : no matching");
	break;
      }
      p->self->state = TASK_DONE;	/* TODO */
    }
  }
}








