/**************************************
 * $Author: kaneda $
 * $Date: 2002/12/27 08:16:06 $
 **************************************/

/* handler.c */

#include "vpgd.h"


static msg_t create_msg(ph_vp_t vp, task_id_t id, msg_type_t type, ...) {
  msg_t msg = NULL;
  va_list ap;
    
  va_start(ap, type);
  switch (type) {
  case ASSUME_VP: {
    int MSG_SIZE = 300;
    char * body;
    ph_vp_t vp1, vp2;

    vp1 = (ph_vp_t)va_arg(ap, ph_vp_t);
    vp2 = (ph_vp_t)va_arg(ap, ph_vp_t);

    body = (char *)Calloc(MSG_SIZE, sizeof(char));
    Mzero(body, MSG_SIZE);
    Snprintf(body, MSG_SIZE, "%lld %lld", vp1, vp2);

    msg = msg_new(vp, id, type, body, Strlen(body) +1);
    break;
  }
  case WRITE_TERMINAL: {
    char * body;
    size_t len;

    body  = (char *)va_arg(ap, char *);
    len = (size_t)va_arg(ap, size_t);

    msg = msg_new(vp, id, type, body, len);
    break;
  }
  case CLOSE_TERMINAL:
    msg = msg_new(vp, id, type, Mdup("CLOSE", 6), 6);
    break;

  case SEND_MACHINE_LIST: {
    ph_pt2pt_t pt2pt;
    ph_routing_entry_cell_t p;
    int nelm, i;
    size_t len;
    char * body;

    /* Note: access the inside of the Phoenix runtime */

    pt2pt = ph_get_pt2pt();
    ph_pthread_mutex_lock(&pt2pt->routing->mp);

    /* get the number of available machines */
    nelm = 0;
    for (p = pt2pt->routing->entry_list->head; p != NULL; p = p->next) {
      if (p->self->metric < PH_MAX_METRIC) {
	nelm++;
      }
    }
    

    len = 100*(nelm+1);
    body = (char *)Calloc(len, sizeof(char));
    Mzero(body, len);

    Snprintf(body, 100, "%d", nelm);

    i = 0;
    for (p = pt2pt->routing->entry_list->head; p != NULL; p = p->next) {
      if (p->self->metric < PH_MAX_METRIC) {
	Snprintf(body + (i+1)*100, 100, "%lld %lld", p->self->dest->lower, p->self->dest->upper);
	i++;
      }
    }
    ph_pthread_mutex_unlock(&pt2pt->routing->mp);

    msg = msg_new(vp, id, type, body, len);
    break;
  }
  default:
    err_quit("send_msg() : no matching\n");
    break;
  }
  va_end(ap);

  assert(msg != NULL);
  return msg;
}


static void handle_local_msg(vpgd_t vpgd, msg_t msg, conn_t conn) {
  assert(msg->id == 0);
  assert(conn != NULL);

  debug_print("handle_local_msg()\t\t:\n");
  

  switch (msg->type) {
  case SPAWN_TASK: {
    task_t task = task_new();
    char s[BUFSIZE];
    int argn;

    Mzero(s, BUFSIZE);
    sscanf(msg->body, "%lld %d %lld %lld %d", 
	   &task->id,
	   (int *)&task->type,
	   &task->out_vp,
	   &task->out_id,
	   &argn);

    task_set_argv(task, msg->body + Strlen(msg->body)+1, argn);
    spawn_task(vpgd, task);
    break;
  }
  case JOIN:
    handle_msg(vpgd, create_msg(conn->vp, 0, ASSUME_VP, vpgd_get_host_vp(vpgd), conn->vp), NULL);
    break;
  case ASSUME_VP:
    //sscanf(msg->body, "%lld ", &vsh->host_vp, vsh->my_vp);
    break;
  case SET_ROUTE:
    sscanf(msg->body, "%d %lld %lld", (int *)&conn->type, &conn->vp, &conn->id);
    break;
  case GET_MACHINE_LIST:
    handle_msg(vpgd, create_msg(conn->vp, 0, SEND_MACHINE_LIST), NULL);
    break;
  default:
    err_quit("handle_local_msg() : no matching");
    break;
  }
  
  msg_dispose(msg);
}


static void handle_remote_msg(vpgd_t vpgd, msg_t msg) {
  conn_t conn;

  debug_print("handle_remote_msg()\t\t:\n");

  /* $B$b$7B>$N2>A[%N!<%I08$N%a%C%;!<%8$J$i$P!$(Bph_send() $B$K$h$C$FE>Aw$9$k(B */
  if (ph_vps_is_member(vpgd->vps, msg->vp) == 0) {
    msg_send(msg, NULL);
    msg_dispose(msg);
    return;
  }

  /* $B:#8=:_3+$$$F$$$J$$%3%M%/%7%g%s08$N%a%C%;!<%8(B ($B$^$@(Bspawn$B$7$F$$$J$$%W%m%;%908(B)$B$J$i$P(B queue $B$K3NJ]$9$k(B */
  if ((conn = conn_list_find_same_dest(vpgd->conns, msg->vp, msg->id)) == NULL) {
    msg_list_enqueue(vpgd->msgs, msg);    
    return;
  }

  switch (conn->type) {
  case SHELL:
    msg_send(msg, conn);
    break;
  case TASK_IN:
    switch (msg->type) {
    case WRITE_TERMINAL:
      if (conn->state == OPEN) {
	msg_send(msg, conn);
      }
      break;
    case CLOSE_TERMINAL:
      conn->state = CLOSED;
      conn_list_delete(vpgd->conns, conn);	/* $B$3$N;~E@$G:o=|$7$F$b2?$NLdBj$b$J$$(B */
      break;
    default:
      err_quit("handle_remote_msg() : no matching\n");
      break;
    }
    break;
  default:
    err_quit("handle_remote_msg() : no matching\n");
    break;
  }

  msg_dispose(msg);
}



static int is_local_msg(vpgd_t vpgd, msg_t msg) {
  return ((msg->vp == -1LL) ||
	  ((msg->vp == vpgd_get_host_vp(vpgd)) && (msg->id == 0)));
}

void handle_msg(vpgd_t vpgd, msg_t msg, conn_t conn) {
  assert(msg != NULL);

  debug_print4("handle_msg()\t\t\t:\n");
  msg_print(msg);

  if (is_local_msg(vpgd, msg)) {
    handle_local_msg(vpgd, msg, conn);    
  } else {
    handle_remote_msg(vpgd, msg);
  }
}


static void delete_conn(vpgd_t vpgd, conn_t conn) {
  switch (conn->type) {
  case SHELL: {
    conn->state = CLOSED;
    break;
  }
  case TASK_OUT:
    if (conn->state != CLOSED) {
      handle_msg(vpgd, create_msg(conn->vp, conn->id, CLOSE_TERMINAL), conn);
      conn->state = CLOSED;
    }
    break;
  default:
    err_quit("delete_conn() : no matching");
      break;
  }
}


static ph_vp_t get_unused_vp(vpgd_t vpgd) {
  ph_vp_t vp;
  conn_cell_t p;
  int i;
  int flag[128];

  for (i = 0; i < 128; i++) {
    flag[i] = 0;
  }
  flag[0] = 1;

  /* TODO */
  for (p = vpgd->conns->head; p != NULL; p = p->next) {
    if (p->self->type == SHELL) {
      flag[(int)(p->self->vp - vpgd_get_host_vp(vpgd))] = 1;
    }
  }

  vp = vpgd_get_host_vp(vpgd);
  for (i = 0; i < 128; i++) {
    if (flag[i] == 0) {
      vp += (ph_vp_t)i;
      break;
    }
  }
  
  assert(vp != vpgd_get_host_vp(vpgd));
  return vp;
}

void handle_accept(vpgd_t vpgd, conn_t conn, fd_set * r_fdset_p) {
  conn_t new;

  if ((conn->fd < 0) || (FD_ISSET(conn->fd, r_fdset_p) == 0)) {
    return;
  }

  if ((new =  conn_accept(conn, get_unused_vp(vpgd))) == NULL) {
    return;
  }
  conn_list_insert(vpgd->conns, new);
}


void handle_recv(vpgd_t vpgd, conn_t conn, fd_set * r_fdset_p) {
  msg_t msg;

  if ((conn->fd < 0) || (FD_ISSET(conn->fd, r_fdset_p) == 0)) {
    return;
  }

  if ((msg = msg_recv(conn)) == NULL) {
    delete_conn(vpgd, conn);
    return;
  }
  
  handle_msg(vpgd, msg, conn);
}


void handle_error(vpgd_t vpgd, conn_t conn, fd_set * e_fdset_p) {
  if ((conn->fd < 0) || (FD_ISSET(conn->fd, e_fdset_p) == 0)) {
    return;
  }

  delete_conn(vpgd, conn);
}










