/**************************************
 * $Author: kaneda $
 * $Date: 2003/03/12 12:04:11 $
 **************************************/

/* main.c */

#include "vpgd.h"



/* check whether config file is available or not before initialize_daemon() 
   in order to print warning */
static void check_conf(char * confname) {
  char * pth;
  FILE * fp;
  
  if (confname != NULL) {
    pth = confname;
  } else if ((pth = getenv("VPG_CONF_PATH")) == NULL) {
    fprintf(stderr, "warning: VPG_CONF_PATH is empty\n");
    return;
  }

  if ((fp = fopen(pth, "r")) == NULL) {
    fprintf(stderr, "warning: cannot open config file \"%s\"\n", pth);
    return;
  }
  fclose(fp);
}

static void check_nickname(char * nickname, int is_test) {
  char s[BUFSIZE];

  Mzero(s, BUFSIZE);
  if (nickname != NULL) {
    Mmove(s, nickname, Strlen(nickname));
  } else {
    gethostname(s, BUFSIZE);
    s[8] = '\0';
  }
  if (is_test) { fprintf(stderr, "nickname=\"%s\"\n\n", s); fflush(stderr); }
  nickname2vp(s);
}

static void check(char * confname, char * nickname, int is_test) {
  check_conf(confname);
  check_nickname(nickname, is_test);
}

static void read_conf(char * confname, int is_test) {
  char * pth;
  FILE * fp;

  char s[PH_BUFSIZE];

  if (confname != NULL) {
    pth = confname;
  } else if ((pth = getenv("VPG_CONF_PATH")) == NULL) {
    ph_err_msg("VPG_CONF_PATH is not set");
    return;
  }

  if ((fp = fopen(pth, "r")) == NULL) {
    ph_err_msg("cannot open config file \"%s\"", pth);
    return;
  }

  if (is_test) { printf("parsing config file \"%s\"\n", pth); fflush(stderr); }
  
  while (fgets(s, PH_BUFSIZE, fp) != NULL) {
    char host[PH_BUFSIZE];
    char port[PH_BUFSIZE];
    char user[PH_BUFSIZE];

    ph_mzero(host, PH_BUFSIZE);
    ph_mzero(port, PH_BUFSIZE);
    ph_mzero(user, PH_BUFSIZE);

    sscanf(s, "%s %s %s", host, port, user);
    ph_add_contact_point(host, atoi(port), ssh, user);
    ph_add_contact_point(host, atoi(port), tcp);
    if (is_test) { printf("  - host=%s, port=%s, user=%s\n", host, port, user); fflush(stderr); }
  }

  fclose(fp);
  if (is_test) { printf("\n"); fflush(stderr); }
}


static void setup_contact_points(char * confname, int is_test) {
  read_conf(confname, is_test);
}


static void initialize_daemon() {
  pid_t pid;
  int i;


  if ((pid = Fork()) != 0) {
    exit(0);
  }

  setsid();

  Signal(SIGHUP, SIG_IGN);

  if ((pid = Fork()) != 0) {
    exit(0);
  }


  //for (i = 0; i < MAX_FD; i++) {
  for (i = 0; i < 3; i++) {
    close(i);
  }
  
  {
    int fd;
    fd = open("/dev/null", O_RDWR);
    assert(fd != -1);
    assert(fd == 0);
    Dup(fd);
    Dup(fd);
  }

}


static void * receiver_th(void * arg) {
  receiver((vpgd_t)arg);
  return NULL;
}


int main(int argn, char ** argv, char ** envp) {
  vpgd_t 	vpgd;

  pthread_t	tid;
  void 		* status;

  int ph_port = DEFAULT_PH_PORT;
  int sh_port = DEFAULT_SH_PORT;
  char * nickname = NULL;
  char * confname = NULL;
  int c;
  int is_test = 0;

  extern char * optarg;
  
  int fds[2];
  
  while ((c = getopt(argn, argv, "p:q:c:n:v:t")) != EOF) {
    switch (c) {
    case 'p':
      ph_port = atoi(optarg);
      break;
    case 'n':
      nickname = Strdup(optarg);
      break;
    case 'c':
      confname = Strdup(optarg);
      break;
    case 't':
      is_test = 1;
      ph_conn_is_test = 1;
      break;
    default:
      printf("Usage: %s [-p <port>] [-c <confname>] [-n <nickname>] [-t] \n", argv[0]);
      exit(1);
    }
  }

  check(confname, nickname, is_test);
  
  if (is_test == 0) { initialize_daemon(); }

  ph_initialize(ph_port);

  setup_contact_points(confname, is_test);

  vpgd = vpgd_new(nickname, sh_port);

  if (nickname != NULL) { Free(nickname); }
  if (confname != NULL) { Free(confname); }
  
  /* setup connection for Phoenix messages */
  {
    Pipe(fds);
    conn_list_insert(vpgd->conns, conn_new(PHOENIX, fds[0], 0LL, 0)); 
  }

  
  Pthread_create(&tid, NULL, &receiver_th, (void *)vpgd);


  while (1) {
    char * s;
    size_t n;
    
    n = ph_recv(&s, 0);
    write(fds[1], s, n);
    Free(s);
  }
  

  Pthread_join(tid, &status);

  ph_finalize(); 

  return 0;
}

