/**************************************
 * $Author: kaneda $
 * $Date: 2002/10/05 09:46:52 $
 **************************************/

/* pt2pt.c */

#include "vpgd.h"


static void initialize_select(vpgd_t vpgd, fd_set * r_fdset_p, fd_set * w_fdset_p, fd_set * e_fdset_p) {
  conn_cell_t p;
  
  FD_ZERO(r_fdset_p);
  FD_ZERO(w_fdset_p);
  FD_ZERO(e_fdset_p);

  for (p = vpgd->conns->head; p != NULL; p = p->next) {
    if (p->self->state != OPEN) {
      continue;
    }

    switch (p->self->type) {
    case LISTENING:
    case SHELL:
    case PHOENIX:
    case TASK_OUT:
      FD_SET(p->self->fd, r_fdset_p);
      FD_SET(p->self->fd, e_fdset_p);
      break;
  case TASK_IN:
    break;
    default:
      err_quit("receive_check_io() : no matching");
      break;
     }
  }
}


static void handle_io(vpgd_t vpgd, conn_t conn, fd_set * r_fdset_p, fd_set * w_fdset_p, fd_set * e_fdset_p) {
  switch (conn->type) {
  case LISTENING:
    handle_accept(vpgd, conn, r_fdset_p);
    break;
  case SHELL:
  case PHOENIX:
  case TASK_OUT:
    handle_recv(vpgd, conn, r_fdset_p);
    break;
  case TASK_IN:
    break;
  default:
    err_quit("handle_io() : no matching");
    break;
  }
  handle_error(vpgd, conn, e_fdset_p);
}


void receiver(vpgd_t vpgd) {
  assert(vpgd != NULL);
 
  conn_list_insert(vpgd->conns, conn_listen(vpgd->listen_port));

  while (1) {
    fd_set r_fdset, w_fdset, e_fdset;
    struct timeval 	timeout;

    conn_cell_t p;

    vpgd_print(vpgd);
 
    initialize_select(vpgd, &r_fdset, &w_fdset, &e_fdset);  

    timeout.tv_sec = RECEIVER_TIMEOUT;
    timeout.tv_usec = 0;
    Select(MAX_FD, &r_fdset, &w_fdset, &e_fdset, &timeout);

    for (p = conn_list_init_loop(vpgd->conns); p != NULL; p = conn_list_iterate(vpgd->conns, p)) {
      handle_io(vpgd, p->self, &r_fdset, &w_fdset, &e_fdset);
    }
    
    release_zombie_tasks(vpgd);
  }
}

