/*************************************
 * $Author: kaneda $
 * $Date: 2003/03/12 09:02:40 $
 **************************************/

/* vpgd.c */

#include "vpgd.h"

static void set_nickname(vpgd_t vpgd, char * nickname) {
   if (nickname != NULL) {
    vpgd->nickname = Strdup(nickname);
  } else {
    char s[BUFSIZE];

    Mzero(s, BUFSIZE);
    gethostname(s, BUFSIZE);
    s[8] = '\0';
    vpgd->nickname = Strdup(s);
  }
}

vpgd_t vpgd_new(char * nickname, int listen_port) {
  vpgd_t vpgd;
  ph_vp_t vp;

  vpgd = (vpgd_t)Malloc(sizeof(struct vpgd));

  set_nickname(vpgd, nickname);
 
  vp = nickname2vp(vpgd->nickname);
  vpgd->vps = ph_vps_make_range(vp, vp+128LL);
  ph_assume_vps(vpgd->vps);

  vpgd->tasks = task_list_new(0);
  vpgd->conns = conn_list_new(0);
  vpgd->msgs = msg_list_new(0);

  vpgd->listen_port = listen_port;

  return vpgd;
}


void vpgd_dispose(vpgd_t vpgd) {
  if (vpgd == NULL) {
    err_msg("vpgd_dispose() : NULL input");
    return;
  }

  free(vpgd->nickname);
  ph_vp_set_dispose(vpgd->vps);

  task_list_dispose(vpgd->tasks);
  conn_list_dispose(vpgd->conns);
  msg_list_dispose(vpgd->msgs);

  Free(vpgd);
}


void vpgd_print(vpgd_t vpgd) {
  assert(vpgd != NULL);

  if (debug_get_mode() == 0) {
    /* to avoid unnecessecities */
    return;
  }

  debug_print2("\n[vpgd ");
  debug_print("nickname=%s\n", vpgd->nickname);
  debug_print2("[vp_set ");
  debug_print("lower=%lld, upper=%lld", ph_vps_get_min_elem(vpgd->vps), ph_vps_get_max_elem(vpgd->vps));
  debug_print2("]\n");
  debug_print("listen_port=%d\n", vpgd->listen_port);


  task_list_print(vpgd->tasks);
  conn_list_print(vpgd->conns);
  msg_list_print(vpgd->msgs);
  debug_print2("]\n\n");
}


/* return its vp that is not allocated to shells */
ph_vp_t vpgd_get_host_vp(vpgd_t vpgd) {
  assert(vpgd != NULL);
  return ph_vps_get_min_elem(vpgd->vps);
}


static int char2abc(int i) {
  int n = 0;

  if ((i >= 48) && (i <= 57)) {		/* [0-9] */
    n = i - 48 + 1;
  } else if ((i >= 65) && (i <= 90)) {	/* [A-Z] */
    n = i - 65 + 11;
  } else if ((i >= 97) && (i <= 122)) {	/* [a-z] */
    n = i - 97 + 37;
  } else {
    err_quit("nickname must be ([a-Z][0-9])*");
  }
  return n;
}



static int abc2char(int i) {
  int n = 0;

  assert((i >= 0) && (i <= 63));
  
  if (i == 0) {
    n = (int)'\0';
  } else if ((i >= 1) && (i <= 10)) {
    n = i -1 + 48;
  } else if ((i >= 11) && (i <= 36)) {
    n = i - 11 + 65;
  } else if ((i >= 37) && (i <= 62)) {
    n = i - 37 + 97;
  } else {
    err_quit("nickname must be [a-Z][0-9]");
  }
  return n;
}


/* this daemon assumes [vp, vp+128) */
ph_vp_t nickname2vp(char * s) {
  ph_vp_t vp;
  int i;


  vp = 0LL;
  for (i = 0; i < min(Strlen(s), 10) ; i++) {
    int j = (int)s[i];
    vp += ((ph_vp_t)char2abc(j)) << (57 - 6*i);
    assert(vp <= PH_MAX_VP);
  }

  return vp;
}


void vp2nickname(ph_vp_t vp, char * s, size_t len) {
  int i;

  assert(len >= 11);
  Mzero(s, len);

  for (i = 0; i < 10; i++) {
    ph_vp_t odd, figure;
    
    figure = 1LL << (57 - 6*i);
    odd = vp % figure;

    s[i] = abc2char((int)((vp - odd) / figure));
    vp = odd;
  }
  s[10] = '\0';
  
  return;
}




